// Package models defines the data models used throughout the application
package models

// TransactionDetail represents detailed information about a transaction
type TransactionDetail struct {
	TransactionID  string                 `json:"transactionId" example:"f973e40540e3b629b7dffd0b91b87aa3474bc31de89e11b01f749bbc5e6d5add"`
	BlockNumber    uint64                 `json:"blockNumber" example:"123"`
	BlockHash      string                 `json:"blockHash" example:"a1b2c3d4e5f6..."`
	Timestamp      string                 `json:"timestamp" example:"2024-01-15T10:30:00Z"`
	ChannelID      string                 `json:"channelId" example:"mychannel"`
	CreatorMSPID   string                 `json:"creatorMspId" example:"Org1MSP"`
	CreatorID      string                 `json:"creatorId" example:"User1@org1.example.com"`
	Endorsers      []string               `json:"endorsers" example:"peer0.org1.example.com,peer0.org2.example.com"`
	ChaincodeID    string                 `json:"chaincodeId" example:"basic"`
	Function       string                 `json:"function" example:"CreateAsset"`
	Arguments      []string               `json:"arguments" example:"asset1,red,10,Alice,1000"`
	Response       TransactionResponse    `json:"response"`
	ValidationCode string                 `json:"validationCode" example:"VALID"`
	RawTransaction map[string]interface{} `json:"rawTransaction,omitempty"`
}

// TransactionResponse represents the response from a transaction
type TransactionResponse struct {
	Status  int32  `json:"status" example:"200"`
	Message string `json:"message" example:"Transaction completed successfully"`
	Payload string `json:"payload" example:""`
}
